@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_FFMPEG_CODECMAP_H__
@@ -41,11 +41,6 @@ gst_ffmpeg_codecid_to_caps   (enum CodecID    codec_id,
  */
 
 GstCaps *
-gst_ffmpeg_codectype_to_caps (enum AVMediaType  codec_type,
-                              AVCodecContext *context, 
-                              enum CodecID codec_id,
-                              gboolean encode);
-GstCaps *
 gst_ffmpeg_codectype_to_audio_caps (AVCodecContext *context, 
                               enum CodecID codec_id,
 				    gboolean encode,
@@ -91,9 +86,15 @@ void
 gst_ffmpeg_videoinfo_to_context (GstVideoInfo *info,
 				 AVCodecContext *context);
 
+void
+gst_ffmpeg_audioinfo_to_context (GstAudioInfo *info,
+				 AVCodecContext *context);
+
 GstVideoFormat gst_ffmpeg_pixfmt_to_videoformat (enum PixelFormat pixfmt);
 enum PixelFormat gst_ffmpeg_videoformat_to_pixfmt (GstVideoFormat format);
 
+GstAudioFormat gst_ffmpeg_smpfmt_to_audioformat (enum AVSampleFormat sample_fmt);
+
 /*
  * _formatid_to_caps () is meant for muxers/demuxers, it
  * transforms a name (ffmpeg way of ID'ing these, why don't
@@ -104,19 +105,6 @@ enum PixelFormat gst_ffmpeg_videoformat_to_pixfmt (GstVideoFormat format);
 GstCaps *
 gst_ffmpeg_formatid_to_caps (const gchar *format_name);
 
-GstVideoFormat
-gst_ffmpeg_pixfmt_to_video_format (enum PixelFormat pix_fmt);
-
-/* Convert a FFMPEG Pixel Format and optional AVCodecContext
- * to a GstCaps. If the context is ommitted, no fixed values
- * for video/audio size will be included in the GstCaps
- *
- * See below for usefullness
- */
-
-GstCaps *
-gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context, enum CodecID codec_id);
-
 /*
  * _formatid_get_codecids () can be used to get the codecIDs
  * (CODEC_ID_NONE-terminated list) that fit that specific
@@ -131,7 +119,7 @@ gst_ffmpeg_formatid_get_codecids (const gchar *format_name,
 
 
 gboolean
-gst_ffmpeg_channel_layout_to_gst (AVCodecContext * context,
+gst_ffmpeg_channel_layout_to_gst (guint64 channel_layout, gint channels,
     GstAudioChannelPosition * pos);
 
 #endif /* __GST_FFMPEG_CODECMAP_H__ */
